                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Authors note:   |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/

#ifndef _POLY_H_
#define _POLY_H_

class Poly {
protected:
	Vertex v[4];
	TextureID base, bump;
	Vertex s,t,normal;
	float scaleS, scaleT;
public:
	Poly(){}

	Poly(Vertex v0, Vertex v1, Vertex v2, TextureID texture, TextureID bumpMap, float scaleFactorS, float scaleFactorT){
		v[0] = v0;
		v[1] = v1;
		v[2] = v1 + v2 - v0;
		v[3] = v2;

		base  = texture;
		bump  = bumpMap;

		scaleS = scaleFactorS;
		scaleT = scaleFactorT;

		s = v1 - v0;
		t = v2 - v0;
		s.normalize();
		t.normalize();
		normal = cross(s,t);
	}

	void drawLit(Renderer *renderer, TextureID vectorField, TextureID distanceMap, const Vertex &pos, const Vertex &lPos){
		glColor3f(1.0f,0.4f,0.1f);
		renderer->setTextures(vectorField, bump, base, distanceMap);
		renderer->setColorTexEnvs(TEXENV_REPLACE, TEXENV_DOT3,     TEXENV_MODULATE, TEXENV_INTERPOLATE_PRIMARY);
		renderer->setAlphaTexEnvs(TEXENV_REPLACE, TEXENV_PASSTHRU, TEXENV_PASSTHRU, TEXENV_REPLACE);
		renderer->apply();

		glBegin(GL_QUADS);
		for (int i = 0; i < 4; i++){
			Vertex vt = (lPos - v[i]) * 0.0006f;
			glMultiTexCoord3fARB(GL_TEXTURE0_ARB, vt * s + 0.5f, vt * t + 0.5f, vt * normal + 0.5f);
			glMultiTexCoord2fARB(GL_TEXTURE1_ARB, scaleS * (i == 1 || i == 2), scaleT * (i >= 2));
			glMultiTexCoord2fARB(GL_TEXTURE2_ARB, scaleS * (i == 1 || i == 2), scaleT * (i >= 2));
			vt = (pos - v[i]) * 0.0003f;
			glMultiTexCoord3fARB(GL_TEXTURE3_ARB, vt * s + 0.5f, vt * t + 0.5f, vt * normal + 0.5f);

			glVertex3fv(v[i]);
		}
		glEnd();
	}

	void drawShadowVolume(const Vertex &pos, const Vertex &lPos){
		Vertex vt, dir;
		int i;

		glBegin(GL_QUADS);
		for (i = 0; i < 4; i++) glVertex3fv(v[i]);
		glEnd();

		glBegin(GL_TRIANGLE_STRIP);
		for (i = 0; i <= 4; i++){
			vt = (pos - v[i & 3]) * 0.0003f;
			glTexCoord3f(vt * s + 0.5f, vt * t + 0.5f, vt * normal + 0.5f);
			glVertex3fv(v[i & 3]);
			dir = v[i & 3] + 1000 * normalize(v[i & 3] - lPos);
			vt = (pos - dir) * 0.0003f;
			glTexCoord3f(vt * s + 0.5f, vt * t + 0.5f, vt * normal + 0.5f);
			glVertex3fv(dir);
		}
		glEnd();

		glBegin(GL_QUADS);
		for (i = 4; i >= 0; i--){
			glVertex3fv(v[i & 3] + 1000 * normalize(v[i & 3] - lPos));
		}
		glEnd();

	}

	bool coversLight(const Vertex &pos, const Vertex &lPos){
		Plane plane(v[0], v[1], v[3]);

		// Find interpolation constant
		float k = plane.distance(pos) / (plane.normal * (pos - lPos));
		if (k < 0 || k > 1) return false;

		Vertex p = pos + k * (lPos - pos);

		unsigned int i, last = 3;
		for (i = 0; i < 4; i++){
			Vector edgeNormal = cross(plane.normal, v[i] - v[last]);

			if (edgeNormal * p - (edgeNormal * v[i]) < 0) return false;

			last = i;
		}

		return true;
	}
};



#endif // _POLY_H_
